from collections import Counter
from functools import lru_cache


@lru_cache()
def find_uniq_count_letters(string):
    assert isinstance(string, str), "should be a string"
    letters_count = Counter(string)
    values = tuple(value for value in letters_count.values() if value == 1)
    return len(values)


def find_uniq_count_from_list(income: list):
    assert isinstance(income, list), "should be a list"
    return list(map(find_uniq_count_letters, income))


def open_file(path: str) -> str:
    with open(path, "r") as file:
        print(file)
        text = file.read()
        file.close()
        return text


def make_list_from_text(text):
    assert isinstance(text,str)
    result = text.split(" ")
    return result


def find_uniq_count_in_text(path: str) -> list:
    text = open_file(path)
    income = make_list_from_text(text)
    return find_uniq_count_from_list(income)






from unittest.mock import patch

import case
from collection_framework import main


def test_main_easy(capsys):
    """Негативный тест с без аргументов"""
    with patch('sys.argv', ['collection_framework.py']):
        from collection_framework import main
        main()
        captured = capsys.readouterr()
        assert captured.out == "Can not start app with no STRING and no FILE LINK variables\n"


@patch('sys.argv', ['collection_framework.py','--string', 'fwfowifjw'])
def test_main_1(capsys):
    """Позитивный тест со строкой"""
    main()
    captured = capsys.readouterr()
    assert captured.out == "3\n"


@patch('sys.argv', ['collection_framework.py', '--file', 'uniq\\file.png'])
@patch('uniq.core.open_file')
def test_main_1(open_mock, capsys):
    """Позитивный тест с файлом 1 аргумент"""
    open_mock.return_value = case.TEXT
    main()
    captured = capsys.readouterr()
    assert captured.out == str(case.result) + "\n"


@patch('sys.argv', ['collection_framework.py', '--file', 'uniq\\file.png', '--string', 'fwfowifjw'])
@patch('uniq.core.open_file')
def test_main_1(open_mock,capsys):
    """Позитивный тест с файлом 2 аргумента"""
    open_mock.return_value = case.TEXT
    main()
    captured = capsys.readouterr()
    assert captured.out == str(case.result) + "\n"


@patch('sys.argv',['collection_framework.py', '--file', 'uniq\\file.png'])
def test_main_2(capsys):
    """Тест файл не найден 1 аргумент"""
    main()
    captured = capsys.readouterr()
    assert captured.out == "Cant to find your file\n"


@patch('sys.argv',['collection_framework.py', '--file', 'uniq\\file.png', '--string', 'fwfowifjw'])
def test_main_3(capsys):
    """Тест файл не найден 2 аргумента"""
    main()
    captured = capsys.readouterr()
    assert captured.out == "Cant to find your file\n"


@patch('sys.argv', ['collection_framework.py', '--file', 'uniq\\file.png', '--string', 'fwfowifjw'])
@patch('uniq.core.open_file')
def test_main_broken_text(open_mock, capsys):
    """Негативный тест """
    open_mock.return_value = b'\x03\x00\x15\x07\nY\x1c\n\x0b\x07\x01'
    main()
    captured = capsys.readouterr()
    assert captured.out == "Sorry we have an unexpected error." + "\n"






from core import find_uniq_count_letters, find_uniq_count_in_text
from parcer import use_parser


def handler_choice(string, path):
    if path:
        return find_uniq_count_in_text(path)
    elif string:
        return find_uniq_count_letters(string)
    else:
        return "Can not start app with no STRING and no FILE LINK variables"


def main():
    string, path = use_parser()
    try:
        answer = handler_choice(string, path)
    except UnicodeDecodeError:
        answer = "Cant to read this file"
    except FileNotFoundError:
        answer = "Cant to find your file"
    except Exception as error:
        answer=f"Sorry we have an unexpected error."
    print(answer)


if __name__ == "__main__":
    main()



from unittest.mock import patch
import pytest
import collection_framework
from core import find_uniq_count_from_list, find_uniq_count_letters, find_uniq_count_in_text
from case import TEXT, result


@pytest.mark.parametrize("income, output", [
    ("asdasdf", 1),
    ("asdasdff", 0),
    ("!", 1),
    ("123321asd4", 4),
    ("z,xcmvnbpfe12", 13)
])
def test_find_count(income, output):
    assert find_uniq_count_letters(income) == output


@pytest.mark.parametrize("example", [
    True,
    122,
    1.22,
    ("223",),
])
def test_find_count_types(example):
    with pytest.raises(AssertionError, match="should be a string"):
        find_uniq_count_letters(example)


@pytest.mark.parametrize("example", [
    True,
    122,
    1.22,
    (0,),
])
def test_find_count_from_list_types(example):
    with pytest.raises(AssertionError, match="should be a list"):
        find_uniq_count_from_list(example)


def test_find_count_in_list():
    strings = [
        "example strings",
        "u can change it and put text",
        "a;lkjfowiangirgegreargasgr",
        "aikmfowijfoiwjgoierjgoisoi",
        "alkdmwifmewoijgmeioprjgeopg",
        "adoijwoifjowigoergoeijgoiejgpajpgforeajgoijsea[g"
        "porijse[ropgijprgjkepgkeporkgspokgpesokg,"
        "[peoksg[poirmbv;psijkt;sp",
        "adoijwoifjowigoergoeijgoiejgpajpgforeajgoijsea["
        "gporijse[ropgijprgjkepgkeporkgspokgpesokg,"
        "[peoksg[poirmbv;psijkt;sp",
        "adoijwoifjowigoergoeijgoiejgpajpgforeajgoijsea"
        "gporijse[ropgijprgjkepgkeporkgspokgpesokg,"
        "[peoksg[poirmbv;psijkt;sp",
        "adoijwoifjowigoergoeijgoiejgpajpgforeajg"
        "oijsea[gporijse[ropgijprgjkepgkeporkgspokgpesokg,"
        "[peoksg[poirmbv;psijkt;sp",
        "adoijwoifjowigoergoeijgoiejg",
        "adoijwoifjowigoergoeijgoiejg",
        "asdasdasdasdjjf"
    ]
    results = [11, 6, 9, 6, 6, 6, 6, 6, 6, 4, 4, 1]
    assert find_uniq_count_from_list(strings) == results


def test_force_easy_working_cashe():
    """
    Кеш точно пустой?
    В кеш точно есть 1 запись?
    При повторной передачи одного и того же значения в кеше все равно 1 запись?
    """
    find_uniq_count_letters.cache_clear()
    assert find_uniq_count_letters.cache_info().currsize == 0
    find_uniq_count_letters('first string')
    assert find_uniq_count_letters.cache_info().currsize == 1
    assert find_uniq_count_letters.cache_info().hits == 0
    assert find_uniq_count_letters.cache_info().misses == 1
    find_uniq_count_letters('first string')
    assert find_uniq_count_letters.cache_info().currsize == 1
    assert find_uniq_count_letters.cache_info().hits == 1
    assert find_uniq_count_letters.cache_info().misses == 1
    find_uniq_count_letters('first string')
    assert find_uniq_count_letters.cache_info().currsize == 1
    assert find_uniq_count_letters.cache_info().hits == 2
    assert find_uniq_count_letters.cache_info().misses == 1
    find_uniq_count_letters('first string')
    assert find_uniq_count_letters.cache_info().currsize == 1
    assert find_uniq_count_letters.cache_info().hits == 3
    assert find_uniq_count_letters.cache_info().misses == 1
    find_uniq_count_letters("second string")
    assert find_uniq_count_letters.cache_info().currsize == 2
    assert find_uniq_count_letters.cache_info().hits == 3
    assert find_uniq_count_letters.cache_info().misses == 2


def test_unic_count_in_text(mocker):
    mocker.patch('src.blackhole.uniq.core.open_file', return_value=TEXT)
    assert find_uniq_count_in_text('any') == result


@pytest.mark.parametrize("string , file, comeback", [
    ("String", None, "one count"),
    (None, "uniq/file.txt", "list of count"),
    (None, None, "Can not start app with no STRING and no FILE LINK variables"),
    ("String", "uniq/file.txt", "list of count")])
def test_start_foo(string, file, comeback, mocker):
    mocker.patch("src.blackhole.collection_framework.find_uniq_count_letters", return_value="one count")
    mocker.patch("src.blackhole.collection_framework.find_uniq_count_in_text", return_value="list of count")
    assert collection_framework.handler_choice(string, file) == comeback


@pytest.mark.parametrize("fake_args, expected_string, expected_path", [
    (['collection_framework.py', '--file', 'uniq\\file.png'], None, 'uniq\\file.png'),
    (['collection_framework.py', '--file', 'uniq\\file.png', '--string', 'fwfowifjw'], 'fwfowifjw', 'uniq\\file.png'),
    (['collection_framework.py', '--string', 'fwfowifjw'], 'fwfowifjw', None), ])
def test_parcer(fake_args, expected_string, expected_path):
    with patch('sys.argv', fake_args):
        from parcer import use_parser
        string, file = use_parser()
        assert string == expected_string
        assert file == expected_path







TEXT = """Lorem ipsum dolor sit amet, consectetur adipiscing elit.
Nullam vestibulum velit non eros consequat aliquam.
Nullam ut risus venenatis, laoreet libero at, tempus libero.
Suspendisse potenti. Suspendisse vel mauris in metus varius
egestas vel id ipsum. Class aptent taciti sociosqu ad litora
torquent per conubia nostra, per inceptos himenaeos.
Proin sed ligula mi. Donec cursus, lacus vel pharetra
pellentesque, nunc turpis tristique turpis, in commodo
neque est sit amet ante. Fusce pulvinar, dui in vehicula
luctus, tellus lectus hendrerit diam, id ultricies neque
magna sit amet nisl. Fusce at vulputate ligula, eu tempor
felis. Duis quis molestie neque. Etiam venenatis libero non
sollicitudin faucibus. Duis tempor tincidunt felis. Vestibulum
interdum neque in risus mattis, eget tempus lacus ultricies.
Aenean et lectus pharetra, fringilla ex sed, interdum orci.
Maecenas ullamcorper rutrum felis, ac congue ante sagittis eget.
Pellentesque diam dolor, consequat nec sagittis eget, scelerisque
sed mi. Fusce vulputate laoreet dui posuere dapibus. Vivamus
vestibulum consectetur ipsum, vel vestibulum tortor congue et.
Aenean pellentesque fringilla erat, ac pellentesque lorem eleifend
id. Quisque neque velit, gravida in tempus ac, commodo id mi.
Nulla vel porttitor nibh. Sed et magna vitae eros dictum ultricies.
Aenean vestibulum a nunc eu fringilla.
Donec sollicitudin hendrerit gravida.
Etiam faucibus elit nec mi mattis, at pellentesque nibh vestibulum.
Vestibulum non faucibus ante. Proin luctus tempus blandit. Sed blandit
pellentesque nisl ut porttitor. Aliquam nec tristique dui. Maecenas erat
dui, interdum interdum tempus id, efficitur a nibh.
Donec ut ex elementum, bibendum turpis at, rhoncus odio. Pellentesque
mollis interdum volutpat. Suspendisse potenti. Sed a leo convallis, porta
felis id, faucibus augue. Phasellus purus odio, suscipit nec interdum sed,
scelerisque a sapien. Fusce ut varius velit, volutpat posuere mauris. Quisque
gravida risus a elit egestas elementum. Integer cursus sapien sed lacus iaculis malesuada.
Curabitur tincidunt feugiat mauris, ultricies volutpat est mollis vel.
Nullam dignissim orci vitae volutpat sagittis. Pellentesque habitant morbi
tristique senectus et netus et malesuada fames ac turpis egestas. Sed vestibulum
hendrerit scelerisque. Sed venenatis, mi a molestie ullamcorper, nisl nunc ullamcorper mi,
feugiat porttitor tortor dolor at dolor. Aliquam sem magna, fermentum imperdiet malesuada eu,
maximus quis arcu. Curabitur auctor leo lacus, sit amet porta sem fringilla sed. Mauris sed augue
vulputate nulla efficitur fringilla. Etiam vestibulum massa sit amet pretium luctus. Ut porta est
ac sodales finibus. Donec congue facilisis odio id mattis."""

result = [5, 5, 3, 3, 5, 5, 7, 9, 8, 5, 1, 4, 9, 5, 2, 3, 6, 5,
          6, 3, 6, 11, 6, 6, 3, 6, 2, 5, 7, 3, 2, 6, 3, 4, 2, 4,
          2, 8, 3, 7, 7, 3, 8, 8, 3, 4, 3, 5, 3, 5, 3, 7, 2, 6, 5,
          7, 2, 6, 3, 3, 4, 5, 5, 9, 3, 2, 9, 4, 6, 5, 5, 2, 7, 5,
          3, 4, 5, 5, 2, 5, 5, 2, 11, 4, 4, 6, 4, 5, 5, 6, 6, 7, 4,
          6, 3, 6, 8, 3, 2, 3, 5, 2, 6, 5, 10, 2, 6, 5, 5, 2, 4, 8,
          8, 7, 2, 6, 2, 6, 4, 2, 8, 4, 4, 9, 3, 2, 3, 8, 3, 5, 5, 5,
          3, 5, 8, 7, 5, 6, 3, 8, 0, 6, 5, 6, 5, 5, 2, 6, 5, 5, 5, 3,
          6, 5, 2, 6, 3, 2, 2, 7, 3, 2, 5, 3, 2, 3, 5, 4, 6, 10, 8, 1,
          2, 2, 10, 7, 5, 9, 6, 4, 3, 2, 5, 2, 6, 4, 4, 1, 6, 5, 5, 4,
          6, 8, 3, 9, 4, 2, 3, 7, 3, 5, 4, 4, 9, 8, 8, 6, 3, 5, 1, 9,
          2, 2, 5, 6, 6, 3, 7, 3, 9, 8, 7, 6, 6, 3, 1, 3, 8, 11, 3, 6,
          4, 5, 3, 3, 4, 3, 8, 9, 1, 7, 5, 2, 6, 6, 6, 5, 7, 9, 3, 1,
          4, 3, 5, 5, 2, 6, 3, 5, 5, 11, 3, 7, 7, 7, 6, 3, 4, 8, 4, 4,
          5, 6, 3, 6, 4, 8, 4, 2, 5, 2, 6, 5, 2, 6, 4, 3, 9, 7, 3, 6, 2,
          1, 6, 8, 4, 2, 7, 9, 2, 0, 3, 2, 4, 7, 3, 4, 5, 5, 6, 7, 4, 5,
          5, 6, 3, 6, 3, 4, 5, 3, 5, 4, 6, 3, 5, 3, 5, 6, 5, 8, 1, 3, 4,
          7, 5, 2, 5, 6, 5, 6, 5, 6, 4, 2, 2, 5]



import argparse


def use_parser():
    parser = argparse.ArgumentParser(description="Find uniq letters from strings")
    parser.add_argument('--string', type=str, default=None, help="Enter your string")
    parser.add_argument('--file', type=str, default=None, help="Enter your file path")
    args = parser.parse_args()
    return args.string, args.file